from abc import ABC, abstractmethod


class ItemStore(ABC):

    @abstractmethod
    def create_item(self, type, name=None, attributes=None):
        pass

    @abstractmethod
    def retrieve_item(self, id):
        pass

    @abstractmethod
    def update_item(self, item):
        pass

    @abstractmethod
    def delete_item(self, id):
        pass

    @abstractmethod
    def find_items(self, query, cursor):
        pass

    @abstractmethod
    def create_type(self, name, items=None, data=None, attributes=None):
        pass

    @abstractmethod
    def retrieve_type(self, id):
        pass

    @abstractmethod
    def update_type(self, type):
        pass

    @abstractmethod
    def delete_type(self, id):
        pass

    @abstractmethod
    def find_types(self, query, cursor):
        pass



import unittest

from realnet_core import ItemMemStore


class MemoryStoreTestCase(unittest.TestCase):

    def test_create_type(self):
        ms = ItemMemStore()

        t1 = ms.create_type('2', attributes={'three': 'four'})
        t2 = ms.retrieve_type(t1.id)

        self.assertEqual(t2.name, '2')
        self.assertEqual(t2.attributes['three'], 'four')

    def test_create_item(self):
        ms = ItemMemStore()

        t1 = ms.create_type('2', attributes={'three': 'four'})
        i1 = ms.create_item(t1, '6', attributes={'seven': 'eight'})

        i2 = ms.retrieve_item(i1.id)

        self.assertEqual(i2.type.name, '2')
        self.assertEqual(i2.type.attributes['three'], 'four')
        self.assertEqual(i2.name, '6')
        self.assertEqual(i2.attributes['seven'], 'eight')



if __name__ == '__main__':
    unittest.main()



from abc import ABC, abstractmethod


class DataStore(ABC):

    @abstractmethod
    def create_data(self, data, attributes=None):
        pass

    @abstractmethod
    def put_data(self, data):
        pass

    @abstractmethod
    def get_data(self, id):
        pass

    @abstractmethod
    def delete_data(self, id):
        pass




from abc import ABC, abstractmethod

class Receiver(ABC):

    @abstractmethod
    def create(self, *args, **kwargs):
        pass

    @abstractmethod
    def put(self, object):
        pass


class ItemReceiver(Receiver):

    def create(self, *args, **kwargs):
        pass

    def put(self, object):
        pass


class TypeReceiver(Receiver):

    def create(self, *args, **kwargs):
        pass

    def put(self, object):
        pass


class DataReceiver(Receiver):

    def create(self, *args, **kwargs):
        pass

    def put(self, object):
        pass


import unittest

from realnet_core import Item, Type


class ItemTestCase(unittest.TestCase):

    def test_dict(self):
        t1 = Type('1', '2', attributes={'three': 'four'})
        i1 = Item('5', '6', t1, attributes={'seven': 'eight'})
        i2 = Item.from_dict(i1.to_dict())

        self.assertEqual(i2.type.id, '1')
        self.assertEqual(i2.type.name, '2')
        self.assertEqual(i2.type.attributes['three'], 'four')
        self.assertEqual(i2.id, '5')
        self.assertEqual(i2.name, '6')
        self.assertEqual(i2.attributes['seven'], 'eight')

    def test_json(self):
        t1 = Type('1', '2', attributes={'three': 'four'})
        i1 = Item('5', '6', t1, attributes={'seven': 'eight'})
        i2 = Item.from_json(i1.to_json())

        self.assertEqual(i2.type.id, '1')
        self.assertEqual(i2.type.name, '2')
        self.assertEqual(i2.type.attributes['three'], 'four')
        self.assertEqual(i2.id, '5')
        self.assertEqual(i2.name, '6')
        self.assertEqual(i2.attributes['seven'], 'eight')

    def test_create_item_from_type_no_override(self):
        t1 = Type('1', '2', attributes={'three': 'four'})
        i1 = t1.create_item('5', '6')

        self.assertEqual(i1.type.id, '1')
        self.assertEqual(i1.type.name, '2')
        self.assertEqual(i1.type.attributes['three'], 'four')
        self.assertEqual(i1.id, '5')
        self.assertEqual(i1.name, '6')

    def test_create_item_from_type_override_attributes(self):
        t1 = Type('1', '2', attributes={'three': 'four', 'ten': 'eleven'})
        i1 = t1.create_item('5', '6',  attributes={'three': 'seven', 'eight': 'nine'})

        self.assertEqual(i1.type.id, '1')
        self.assertEqual(i1.type.name, '2')
        self.assertEqual(i1.type.attributes['three'], 'four')
        self.assertEqual(i1.id, '5')
        self.assertEqual(i1.name, '6')
        self.assertEqual(i1.attributes['three'], 'seven')
        self.assertEqual(i1.attributes['eight'], 'nine')
        self.assertEqual(i1.attributes['ten'], 'eleven')

    def test_create_item_from_type_with_children(self):
        t1 = Type('1', '2', attributes={'three': 'four', 'ten': 'eleven'})
        i1 = t1.create_item('5', '6', {'three': 'seven', 'eight': 'nine'})

        self.assertEqual(i1.type.id, '1')
        self.assertEqual(i1.type.name, '2')
        self.assertEqual(i1.type.attributes['three'], 'four')
        self.assertEqual(i1.id, '5')
        self.assertEqual(i1.name, '6')
        self.assertEqual(i1.attributes['three'], 'seven')
        self.assertEqual(i1.attributes['eight'], 'nine')
        self.assertEqual(i1.attributes['ten'], 'eleven')



if __name__ == '__main__':
    unittest.main()



from .type import Type
import json
import copy


class Item:
    def __init__(self, id, name, type, attributes={}):
        self.id = id
        self.name = name
        self.type = type
        self.attributes = dict(list(copy.deepcopy(self.type.attributes).items()) + list(attributes.items()))

    def to_dict(self):
        return {'id': self.id,
                'name': self.name,
                'type': self.type.to_dict(),
                'attributes': self.attributes if isinstance(self.attributes, dict) else {}}

    def to_json(self):
        return json.dumps(self.to_dict())

    def __hash__(self):
        return hash(self.id)

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            # Ignoring .b attribute
            return self.id == other.id
        else:
            return NotImplemented

    def __repr__(self):
        return "Item(%s, %s, %s, %s)" % (self.id, self.name, self.type.__repr__(), self.attributes)

    @classmethod
    def from_dict(cls, data):
        return Item(data['id'],
                    data['name'],
                    Type.from_dict(data['type']),
                    data['attributes'])

    @classmethod
    def from_json(cls, data):
        t = json.loads(data)
        return Item(t['id'], t['name'], Type.from_dict(t['type']), t['attributes'])


import unittest

from realnet_core import DataMemStore


class DataStoreTestCase(unittest.TestCase):

    def test_create_data(self):

        ds = DataMemStore()

        t1 = ds.create_data('2', attributes={'three': 'four'})
        t2 = ds.get_data(t1.id)

        self.assertEqual(t2.data, '2')
        self.assertEqual(t2.attributes['three'], 'four')


if __name__ == '__main__':
    unittest.main()



from abc import ABC, abstractmethod


class Transformer(ABC):

    def __init__(self, func):
        self.func = func

    def pre_transform(self, *args, **kwargs):
        pass

    @abstractmethod
    def transform(self, *args, **kwargs):
        pass

    def post_transform(self, *args, **kwargs):
        pass

    def __call__(self, *args, **kwargs):
        self.pre_transform(args, kwargs)
        result = self.transform(args, kwargs)
        self.post_transform(args, kwargs)
        return result


class ScriptTransformer(Transformer):

    def transform(self, *args, **kwargs):
        pass


class TypeTransformer(Transformer):

    def transform(self, *args, **kwargs):
        pass


class DataTransformer(Transformer):

    def transform(self, *args, **kwargs):
        pass


import pathlib
from setuptools import find_packages, setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="realnet-core",
    version="0.0.9",
    description="Realnet core",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/virtual-space/realnet-core",
    author="Marko Laban",
    author_email="marko.laban@l33tsystems.com",
    license="BSD",
    classifiers=[
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    packages=["realnet_core"],
    include_package_data=True,
    install_requires=[],
    entry_points={
        "console_scripts": []
    },
)



import json

import realnet_core.item


class Type:

    def __init__(self, id, name, items=[], attributes=None):
        self.id = id
        self.name = name
        self.items = items
        self.attributes = attributes

        if id is None and items:
            self.id = items.__hash__()


    def __hash__(self):
        return hash(self.id)

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            # Ignoring .b attribute
            return self.id == other.id
        else:
            return NotImplemented

    def __repr__(self):
        if self.name and (self.items or self.attributes):
            if self.items and self.attributes:
                return "Type(%s, %s, %s, %s)" % (self.id, self.name, [item.__repr__() for item in self.items], self.attributes)
            elif self.items:
                return "Type(%s, %s, %s)" % (self.id, self.name, [item.__repr__() for item in self.items])
            else:
                return "Type(%s, %s, %s)" % (self.id, self.name, self.attributes)
        elif self.name:
            return "Type(%s, %s)" % (self.id, self.name)
        else:
            return "Type(%s)" % (self.id)


    def create_item(self, id, name, attributes=None):
        return realnet_core.item.Item(id,
                                      name,
                                      self,
                                      attributes if attributes else {})

    def to_dict(self):
        return {'id': self.id,
                'name': self.name,
                'items': [item.to_dict() for item in self.items],
                'attributes': self.attributes if isinstance(self.attributes, dict) else {}}

    def to_json(self):
        return json.dumps(self.to_dict())

    @classmethod
    def from_dict(cls, data):
        return Type(data['id'],
                    data['name'],
                    [realnet_core.item.Item.from_dict(item) for item in data['items']],
                    data['attributes'])

    @classmethod
    def from_json(cls, data):
        t = json.loads(data)
        return Type(t['id'],
                    t['name'],
                    [realnet_core.item.Item.from_dict(item) for item in t['items']],
                    t['attributes'])

    # def myFun(arg1, arg2, arg3):
    #     print("arg1:", arg1)
    #     print("arg2:", arg2)
    #     print("arg3:", arg3)
    # args = ("Geeks", "for", "Geeks")
    # myFun(*args)
    # kwargs = {"arg1": "Geeks", "arg2": "for", "arg3": "Geeks"}
    # myFun(**kwargs)

    @classmethod
    def set(cls, *args):
        pass


from .type import Type
from .item import Item
from .itemstore import ItemStore

import os
import uuid
import json


class ItemMemStore(ItemStore):

    def __init__(self, types={}, items={}):
        self.types = types
        self.items = items

    def create_type(self, name, items=None, data=None, attributes={}):
        references = []
        if items:
            references = [existing for existing in [self.items.get(item.id) for item in items] if existing]
        id = str(uuid.uuid4())
        new = Type(id, name, references, attributes)
        self.types[id] = new
        return new

    def retrieve_type(self, id):
        return self.types.get(id)

    def update_type(self, type):

        if type is None:
            return None

        self.types[type.id] = type
        return type

    def delete_type(self, id):
        return self.types.pop(id)

    def find_types(self, query, cursor):
        return None

    def create_item(self, type, name=None, attributes={}):

        if type is None:
            return None

        id = str(uuid.uuid4())
        new = Item(id, name if name else type.name, type, attributes)
        self.items[id] = new

        return new

    def retrieve_item(self, id):
        return self.items.get(id)

    def update_item(self, item):
        if item is None:
            return None

        self.items[item.id] = item

        return item

    def delete_item(self, id):
        return self.items.pop(id)

    def find_items(self, query, cursor):
        return None

    def save(self, path):
        out_types = {t[0]: {'id': t[1]['id'],
                            'name': t[1]['name'],
                            'items': [i['id'] for i in t[1]['items']],
                            'attributes': t[1]['attributes']} for t in self.types.items()}
        out_items = {i[0]: {'id': i[1]['id'],
                            'name': i[1]['name'],
                            'type': i[1]['type']['id'],
                            'attributes': i[1]['attributes']} for i in self.items.items()}
        with open(path, 'w') as outfile:
            json.dump({'types': out_types, 'items': out_items, outfile)

    @classmethod
    def load(cls, path):
        if os.path.exists(path):
            with open(path, 'r') as json_file:
                data = json.load(json_file)
                in_types = {t[0]: Type(t[1]['id'],
                                       t[1]['name'],
                                       t[1]['items'],
                                       t[1]['attributes']) for t in data['types'].items()}
                in_items = {i[0]: Item(i[1]['id'],
                                       i[1]['name'],
                                       i[1]['type'],
                                       i[1]['attributes']) for i in data['items'].items()}
                for item in in_items:
                    in_items[item].type = in_types[item]

                for type in in_types:
                    in_types[type].items = [in_items[item] for item in in_types[type].items]

                return ItemMemStore(in_types, in_items)
        else:
            return ItemMemStore()

